// tctrl.c
//
// John Findlay
//
//
#include <windows.h>
#include <stdlib.h>
#include "tctrl.h"

#define BARWIDTH  10

static HINSTANCE hInst = NULL;
static HPEN hPenShadow, hPenHighLite, hPenBlack;
static char szVertClass[] = "vert";
static char szHorzClass[] = "horz";

LRESULT CALLBACK VertBarWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK HorzBarWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

#define GETSTRUCT(hWnd)	(LPTCTRL)GetWindowLong(hWnd, GWL_USERDATA)

BOOL InitTCrtl(void)
{
	WNDCLASS wc;

	// Get our instance handle
	hInst = GetModuleHandle(NULL);

	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= (WNDPROC)VertBarWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInst;
	wc.hbrBackground	= (HBRUSH)(COLOR_3DFACE+1);
	wc.lpszMenuName		= NULL;
	wc.hIcon			= NULL;
	wc.hCursor			= LoadCursor(NULL, IDC_SIZEWE);
	wc.lpszClassName	= szVertClass;

	if (!RegisterClass(&wc)){
		return FALSE;
	}

	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= (WNDPROC)HorzBarWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInst;
	wc.hbrBackground	= (HBRUSH)(COLOR_3DFACE+1);
	wc.lpszMenuName		= NULL;
	wc.hIcon			= NULL;
	wc.hCursor			= LoadCursor(NULL, IDC_SIZENS);
	wc.lpszClassName	= szHorzClass;

	if (!RegisterClass(&wc)){
		return FALSE;
	}

	return TRUE;

}

HWND CreateTCtrl(HWND hWndParent, int XVert, int YHorz)
{
	RECT rc;

	if (hInst == NULL)		// TCtrl has not be initialised
		return (HWND)0;

	// A few checks
	if (hWndParent == NULL)	// OPPS fogort the parent
		return (HWND)0;

	if (XVert < 10)
		XVert = 10;

    if (YHorz < 10)
		YHorz = 10;

	GetClientRect(hWndParent, &rc);

	HWND hWndVert = CreateWindow(szVertClass, szVertClass,
		WS_CHILD, 0, 0, 0, 0,
		hWndParent, NULL, hInst,
		(LPVOID)XVert);	// pass the start position to the CREATESTRUCT in lParam->lpCreateParams

	if (!hWndVert)
		return (HWND)0;

	HWND hWndHorz = CreateWindow(szHorzClass, szHorzClass,
		WS_CHILD, 0, 0, 0, 0,
		hWndParent, NULL, hInst,
		(LPVOID)YHorz);	// pass the start position to the CREATESTRUCT in lParam->lpCreateParams

	// Get the Vertical window's struct and store the horz window's handle
	// as hwndSibling. The horz window can get the sibling value of the vert
	// window as it is created using FindWindowEx()
	LPTCTRL pms = (LPTCTRL)GetWindowLong(hWndVert, GWL_USERDATA);
	pms->hwndSibling = hWndHorz;

	if (!hWndHorz)
		return (HWND)0;

	// create three pens for drawing the edges of the bar windows
	hPenShadow 	 = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DSHADOW));
	hPenHighLite = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DHIGHLIGHT));
	hPenBlack 	 = CreatePen(PS_SOLID, 1, RGB(0, 0, 0));

	ShowWindow(hWndVert, SW_NORMAL);
	ShowWindow(hWndHorz, SW_NORMAL);
	UpdateWindow(hWndVert);
	UpdateWindow(hWndHorz);

	return hWndVert;
}

LRESULT CALLBACK VertBarWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg){
	case WM_CREATE:{
		/* Allocate memory for TCTRL struct and fill in various values.
		   Finally save the pointer in the window's GWL_USERDATA */
		RECT rc;
		HWND hwp = GetParent(hWnd);
		int XVert = (long)((LPCREATESTRUCT) lParam)->lpCreateParams;

		LPTCTRL pms = malloc(sizeof(TCTRL));
		GetClientRect(hwp, &rc);

		pms->hwndParent = hwp;

		// set members of TCTRL
		pms->x	   = XVert; 				 	// x pos
		pms->y	   = 0;							// y pos
		pms->cx	   = BARWIDTH;					// width
		pms->cy	   = rc.bottom;  				// height

		// save pointer to the TCTRL struct
		SetWindowLong(hWnd, GWL_USERDATA, (long)pms);
		}
		return 0;

	case WM_TCTRL_VERTPOS:{
		LPTCTRL pms = GETSTRUCT(hWnd);
		return pms->x;
		}

	case WM_TCTRL_HORZPOS:{
		LPTCTRL pms  = GETSTRUCT(hWnd);
		LPTCTRL pms1 = GETSTRUCT(pms->hwndSibling);
		return pms1->y;
	}

	case WM_LBUTTONDOWN:
		SetCapture(hWnd);
		return 0;

	case WM_SIZE:{
		RECT rc;
		LPTCTRL pms  = GETSTRUCT(hWnd);
		GetClientRect(pms->hwndParent, &rc);
		pms->y	   = 0;								// y pos
		pms->cx	   = BARWIDTH;						// width
		pms->cy	   = rc.bottom;  					// height
		MoveWindow(hWnd, pms->x, pms->y, pms->cx, pms->cy, TRUE);
		return 0;
		}

	case WM_MOUSEMOVE:
		if (wParam == MK_LBUTTON){
			RECT rc;
			LPTCTRL pms  = GETSTRUCT(hWnd);
			POINT pt;
			GetClientRect(GetParent(hWnd), &rc);
			GetCursorPos(&pt);
			ScreenToClient(pms->hwndParent, &pt);
			if (pt.x > 4 && pt.x < (rc.right-4)){
				pms->x	= pt.x - BARWIDTH/2;
				pms->cx	= BARWIDTH;
				MoveWindow(hWnd, pms->x, pms->y, pms->cx, pms->cy, TRUE);
				SendMessage(pms->hwndSibling , WM_SIZE, 0, 0);
				InvalidateRect(hWnd, NULL, TRUE);
			}
		}
		return 0;

	case WM_PAINT:{
		PAINTSTRUCT ps;
		BeginPaint(hWnd, &ps);
		DrawVertBar(hWnd, &ps);
		EndPaint(hWnd, &ps);
		}
		return 0;

	case WM_LBUTTONUP:
		ReleaseCapture();
		return 0;

	case WM_CLOSE:{
		LPTCTRL pms  = GETSTRUCT(hWnd);
		PostMessage(pms->hwndSibling, WM_CLOSE, 0, 0);
		free((LPVOID)GetWindowLong(hWnd, GWL_USERDATA));
		DestroyWindow(hWnd);
		DeleteObject(hPenHighLite);
		DeleteObject(hPenShadow);
		DeleteObject(hPenBlack);
		return 0;
		}
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK HorzBarWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg){
	case WM_CREATE:{
		/* Allocate memory for TCTRL struct and fill in various values.
		   Retrieve the sibling with FindWindowEx().
		   Finally save the pointer in the window's GWL_USERDATA */
		RECT rc;
		HWND hwp = GetParent(hWnd);
		int YHorz = (long)((LPCREATESTRUCT) lParam)->lpCreateParams;

		LPTCTRL pms = malloc(sizeof(TCTRL));
		GetClientRect(hwp, &rc);
		pms->hwndParent = hwp;

		// Find vert window
		HWND h  = FindWindowEx(hwp, NULL, szVertClass, szVertClass);
		LPTCTRL pms1 = (LPTCTRL)GetWindowLong(h, GWL_USERDATA);
		pms->hwndSibling = h;

		pms->x	   = 0;							  	// x pos
		pms->y	   = YHorz;							// y pos
		pms->cx	   = pms1->x;						// width
		pms->cy	   = BARWIDTH+1;  					// height

		// save pointer to the TCTRL struct
		SetWindowLong(hWnd, GWL_USERDATA, (long)pms);
		}
		return 0;

	case WM_LBUTTONDOWN:
		SetCapture(hWnd);
		return 0;

	case WM_SIZE:{
		RECT rc;
		LPTCTRL pms  = GETSTRUCT(hWnd);
		LPTCTRL pms1 = GETSTRUCT(pms->hwndSibling);

		GetClientRect(pms->hwndParent, &rc);
		pms->x	   = 0;							  	// x pos
		pms->cx	   = pms1->x;		  				// height

		MoveWindow(hWnd, pms->x, pms->y, pms->cx, pms->cy, TRUE);
		return 0;
	}

	case WM_MOUSEMOVE:
		if (wParam == MK_LBUTTON){
			RECT rc;
			LPTCTRL pms  = GETSTRUCT(hWnd);
			LPTCTRL pms1 = GETSTRUCT(pms->hwndSibling);
			POINT pt;
			GetClientRect(GetParent(hWnd), &rc);
			GetCursorPos(&pt);
			ScreenToClient(pms->hwndParent, &pt);
			if (pt.y > 4 && pt.y < (rc.bottom-4)){
				pms->y	= pt.y - BARWIDTH/2-1;
				pms->cx	= pms1->x;
				MoveWindow(hWnd, 0, pms->y, pms->cx, pms->cy, TRUE);
			}
		}
		return 0;

	case WM_PAINT:{
		PAINTSTRUCT ps;
		BeginPaint(hWnd, &ps);
		DrawHorzBar(hWnd, &ps);
		EndPaint(hWnd, &ps);
		}
		return 0;

	case WM_LBUTTONUP:
		ReleaseCapture();
		return 0;

	case WM_CLOSE:
		free((LPVOID)GetWindowLong(hWnd, GWL_USERDATA));
		DestroyWindow(hWnd);
		return 0;
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

void DrawVertBar(HWND hWndVert, PAINTSTRUCT *ps)
{
	SelectObject(ps->hdc, hPenHighLite);
	MoveToEx(ps->hdc, ps->rcPaint.left + 1, ps->rcPaint.top, NULL);
	LineTo(ps->hdc, ps->rcPaint.left + 1, ps->rcPaint.bottom);
	SelectObject(ps->hdc, hPenShadow);
	MoveToEx(ps->hdc, ps->rcPaint.right - 2, ps->rcPaint.top, NULL);
	LineTo(ps->hdc, ps->rcPaint.right - 2, ps->rcPaint.bottom);
	SelectObject(ps->hdc, hPenBlack);
	MoveToEx(ps->hdc, ps->rcPaint.right - 1, ps->rcPaint.top, NULL);
	LineTo(ps->hdc, ps->rcPaint.right - 1, ps->rcPaint.bottom);
}

void DrawHorzBar(HWND hWndVert, PAINTSTRUCT *ps)
{
	SelectObject(ps->hdc, hPenHighLite);
	MoveToEx(ps->hdc, ps->rcPaint.left, ps->rcPaint.top + 1, NULL);
	LineTo(ps->hdc, ps->rcPaint.right, ps->rcPaint.top + 1);
	SelectObject(ps->hdc, hPenShadow);
	MoveToEx(ps->hdc, ps->rcPaint.left, ps->rcPaint.bottom - 2, NULL);
	LineTo(ps->hdc, ps->rcPaint.right, ps->rcPaint.bottom - 2);
	SelectObject(ps->hdc, hPenBlack);
	MoveToEx(ps->hdc, ps->rcPaint.left, ps->rcPaint.bottom - 1, NULL);
	LineTo(ps->hdc, ps->rcPaint.right, ps->rcPaint.bottom - 1);
}

