// wmain.c
//
// Demonstrates a small custom control, TCtrl.
//
// John Findlay
//
//
#include <windows.h>
#include <stdio.h>
#include "tctrl.h"
#include "resource.h"

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

char m[] = "Two child windows as horz and vert bars on main window!";

// The handle for the control.
HWND g_hWndTCtrl;

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmdLine, int nCmdShow)
{

	MSG					msg;
	WNDCLASS			wc;
	HWND				hWnd;

	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= (WNDPROC)MainWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(1));
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName		= MAKEINTRESOURCE(ID_MENU);
	wc.lpszClassName	= "main";

	if (!RegisterClass(&wc))
		return 0;

	hWnd = CreateWindowEx(WS_EX_CLIENTEDGE, "main", "TCtrl Demo...",
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,CW_USEDEFAULT,
		CW_USEDEFAULT,CW_USEDEFAULT,
		NULL,
		(HMENU)0,
		hInstance,
		NULL);

	if (hWnd == NULL)
	{
		MessageBox(NULL, "Not today!", "Message", MB_OK);
		return 0;
	}

	ShowWindow(hWnd, SW_NORMAL);
	UpdateWindow(hWnd);

	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message){

	case WM_CREATE:{
		RECT rc;
		GetClientRect(hWnd, &rc);

		// Init the custom control
		if (!InitTCrtl()){
			MessageBox(0, "Opps!", "Message", MB_OK);
			return -1;
		}

		// Create control
		g_hWndTCtrl = CreateTCtrl(hWnd, 200, 200);

		// Check for the TCtrl window
		if (g_hWndTCtrl == (HWND)0){
			MessageBox(0, "Opps!", "Message", MB_OK);
			return -1;
		}

		// else return ok
		return 0;
	}

	case WM_COMMAND:
		switch(LOWORD(wParam)){
		char m[90];
		case IDM_EXIT:
			PostMessage(hWnd, WM_CLOSE, 0, 0);
			break;

		// A few checks are carried out before trying to access the control
		// if (g_hWndTCtrl)
		case IDM_TCTRL_HORZPOS:
			if (g_hWndTCtrl){
				sprintf(m, "Horz bar position: %d",
						SendMessage(g_hWndTCtrl, WM_TCTRL_HORZPOS, 0, 0));
				MessageBox(hWnd, m, "Message", MB_OK);
			}
			break;

		case IDM_TCTRL_VERTPOS:
			if (g_hWndTCtrl){
				sprintf(m, "Vert bar position: %d",
						SendMessage(g_hWndTCtrl, WM_TCTRL_VERTPOS, 0, 0));
				MessageBox(hWnd, m, "Message", MB_OK);
			}
			break;

		case IDM_TCTRL_CREATE:
			if (!g_hWndTCtrl){
				// Create control
				g_hWndTCtrl = CreateTCtrl(hWnd, 200, 200);
			}
			break;

		case IDM_TCTRL_CLOSE:
			if (g_hWndTCtrl){
				SendMessage(g_hWndTCtrl, WM_CLOSE, 0, 0);
				g_hWndTCtrl = NULL;
			}
			break;
		}
		break;

	case WM_PAINT:{
		PAINTSTRUCT ps;
		BeginPaint(hWnd, &ps);
		TextOut(ps.hdc, 50, 50, m,	strlen(m));
		EndPaint(hWnd, &ps);
		}
		return 0;

	case WM_SIZE:
		SendMessage(g_hWndTCtrl, WM_SIZE, 0, 0);
		return 0;

	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}
